-- Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY LINEIN_CTRL IS
	PORT	(
		PCLK	: IN std_logic;
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
		LB_WR_ADDR	: IN std_logic_vector(9 downto 0);
		LB_WR_DATA	: IN std_logic_vector(15 downto 0);
		LB_WR_N	: IN std_logic;
		CamLineCount : in std_logic_vector(8 downto 0); 
		CamLineCount_sync : out std_logic_vector(8 downto 0); 
		VgaLineCount : in std_logic_vector(8 downto 0);
		VgaPixCount : in std_logic_vector(9 downto 0);
		VgaHsync_edge : in std_logic;
		buf_RGB : out std_logic_vector(15 downto 0)
 );
END LINEIN_CTRL;

ARCHITECTURE structure OF LINEIN_CTRL IS

component SRAM_1024x16
   Port (
          WRCLK : In std_logic;
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(9 downto 0);
          RDADDR : In std_logic_vector(9 downto 0);
          WRDATA : In std_logic_vector(15 downto 0);
          RDDATA : Out std_logic_vector(15 downto 0) );
end component;

 signal LB_RD_ADDR : std_logic_vector(9 downto 0);
 signal LB_RD_DATA_0 : std_logic_vector(15 downto 0);
 signal LB_RD_DATA_1 : std_logic_vector(15 downto 0);
 signal LB_RD_DATA_2 : std_logic_vector(15 downto 0);
 signal LB_RD_DATA_3 : std_logic_vector(15 downto 0);
 signal LB_WR_N_0 : std_logic;
 signal LB_WR_N_1 : std_logic;
 signal LB_WR_N_2 : std_logic;
 signal LB_WR_N_3 : std_logic;
 signal LB_SEL_W, LB_SEL_R : std_logic_vector(1 downto 0);

BEGIN

  LB_SEL_W <= CamLineCount(1 downto 0);

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     LB_SEL_R <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     if(VgaHsync_edge = '1') then -- 2023/06/22
--     if(VgaHsync_edge = '1' and VgaLineCount(1 downto 0) = "00") then
--     if(VgaHsync_edge = '1' and VgaLineCount(0) = '0') then
       LB_SEL_R <= LB_SEL_W - 2;
     end if;
   end if;  
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     CamLineCount_sync <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     if(VgaHsync_edge = '1') then -- 2023/06/22
       CamLineCount_sync <= CamLineCount - 2;
     end if;
   end if;  
end process;

-------------------------------------------------------------------
LineBuf_0 : SRAM_1024x16 port map (
   WRCLK		=> PCLK,
   RDCLK		=> CLK,
   WR_N	=> LB_WR_N_0, 
   WRADDR	=> LB_WR_ADDR, 
   RDADDR	=> LB_RD_ADDR, 
   WRDATA	=> LB_WR_DATA, 
   RDDATA	=> LB_RD_DATA_0 );

-------------------------------------------------------------------
LineBuf_1 : SRAM_1024x16 port map (
   WRCLK		=> PCLK,
   RDCLK		=> CLK,
   WR_N	=> LB_WR_N_1, 
   WRADDR	=> LB_WR_ADDR, 
   RDADDR	=> LB_RD_ADDR, 
   WRDATA	=> LB_WR_DATA, 
   RDDATA	=> LB_RD_DATA_1 );

-------------------------------------------------------------------
LineBuf_2 : SRAM_1024x16 port map (
   WRCLK		=> PCLK,
   RDCLK		=> CLK,
   WR_N	=> LB_WR_N_2, 
   WRADDR	=> LB_WR_ADDR, 
   RDADDR	=> LB_RD_ADDR, 
   WRDATA	=> LB_WR_DATA, 
   RDDATA	=> LB_RD_DATA_2 );

-------------------------------------------------------------------
LineBuf_3 : SRAM_1024x16 port map (
   WRCLK		=> PCLK,
   RDCLK		=> CLK,
   WR_N	=> LB_WR_N_3, 
   WRADDR	=> LB_WR_ADDR, 
   RDADDR	=> LB_RD_ADDR, 
   WRDATA	=> LB_WR_DATA, 
   RDDATA	=> LB_RD_DATA_3 );

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     buf_RGB <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     if(LB_SEL_R = "00") then
       buf_RGB <= LB_RD_DATA_0;
     elsif(LB_SEL_R = "01") then
       buf_RGB <= LB_RD_DATA_1;
     elsif(LB_SEL_R = "10") then
       buf_RGB <= LB_RD_DATA_2;
     else
       buf_RGB <= LB_RD_DATA_3;
     end if;
   end if;  
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		LB_RD_ADDR <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		LB_RD_ADDR <= VgaPixCount; 
	end if;  
end process;

-------------------------------------------------------------------
  LB_WR_N_0 <= LB_WR_N when LB_SEL_W = "00" else '1'; 
  LB_WR_N_1 <= LB_WR_N when LB_SEL_W = "01" else '1'; 
  LB_WR_N_2 <= LB_WR_N when LB_SEL_W = "10" else '1'; 
  LB_WR_N_3 <= LB_WR_N when LB_SEL_W = "11" else '1'; 

END structure;
