-- Designed by Toshio Iwata at DIGITALFILTER.COM 2023/03/15

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY LINEOUT_CTRL IS
	PORT	(
		CLK	: IN std_logic;
		CLK100M	: IN std_logic;
		RST_N	: IN std_logic;
  TsLineCount : out std_logic_vector(8 downto 0);
  TsPixCount : out std_logic_vector(9 downto 0);
                CnnBlockH : in std_logic_vector(4 downto 0);
                CnnBlockV : in std_logic_vector(3 downto 0);
VgaHsync_edge : in std_logic;
RAM_X0_RD_ADDR : in std_logic_vector(9 downto 0);
                ts_r_8bit : out std_logic_vector(7 downto 0);
                ts_g_8bit : out std_logic_vector(7 downto 0);
                ts_b_8bit : out std_logic_vector(7 downto 0);
		latch0	: IN std_logic_vector(15 downto 0);
		latch1	: IN std_logic_vector(15 downto 0);
		latch2	: IN std_logic_vector(15 downto 0);
		latch3	: IN std_logic_vector(15 downto 0);
		VgaLineCount	: in std_logic_vector(8 downto 0);
		VgaLineCountDiv10 : in std_logic_vector(5 downto 0);
		VgaLineCountMod10 : in std_logic_vector(3 downto 0);
		VgaPixCount : IN std_logic_vector(9 downto 0);
		winp0: in std_logic;
		winp1: in std_logic;
		winp2: in std_logic;
                winp3: in std_logic;
   vga_r : out std_logic_vector(7 downto 0);
   vga_g : out std_logic_vector(7 downto 0);
   vga_b : out std_logic_vector(7 downto 0) );
END LINEOUT_CTRL;

ARCHITECTURE structure OF LINEOUT_CTRL IS

component SRAM_1024x16
   Port (
          WRCLK : In std_logic;
          RDCLK : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(9 downto 0);
          RDADDR : In std_logic_vector(9 downto 0);
          WRDATA : In std_logic_vector(15 downto 0);
          RDDATA : Out std_logic_vector(15 downto 0) );
end component;

 signal RAM_X0_WR_ADDR : std_logic_vector(9 downto 0);
 signal RAM_X0_WR_DATA : std_logic_vector(15 downto 0);
 signal RAM_X0_RD_DATA_0 : std_logic_vector(15 downto 0);
 signal RAM_X0_RD_DATA_1 : std_logic_vector(15 downto 0);
 signal RAM_X0_RD_DATA : std_logic_vector(15 downto 0);
 signal RAM_X0_RD_SEL : std_logic;
 signal RAM_X0_WR_N_0 : std_logic;
 signal RAM_X0_WR_N_1 : std_logic;
 signal RAM_X0_WR_SEL : std_logic;
signal ODDLINE, oddline_dly1, oddline_dly2, oddline_dly3, oddline_edge : std_logic;
 signal count4wraddr : std_logic_vector(10 downto 0); -- 9 to 10 for vgabuf 2023/06/24
signal WRN2DL_sig : std_logic;
signal TsPixCount_sig : std_logic_vector(9 downto 0); 
signal TsLineCount_sig : std_logic_vector(8 downto 0); 
signal CnnBlockH_m32 : std_logic_vector(9 downto 0); 
signal CnnBlockV_m30 : std_logic_vector(8 downto 0); 
signal vga_RGB : std_logic_vector(15 downto 0); 
 signal OB_WR_ADDR : std_logic_vector(9 downto 0);
 signal OB_RD_ADDR : std_logic_vector(9 downto 0);
 signal OB_WR_DATA : std_logic_vector(15 downto 0);
 signal OB_RD_DATA_A : std_logic_vector(15 downto 0);
 signal OB_RD_DATA_B : std_logic_vector(15 downto 0);
 signal OB_WR_N_A : std_logic;
 signal OB_WR_N_B : std_logic;
-- signal vga_r_tmp, vga_g_tmp, vga_b_tmp : std_logic_vector(7 downto 0);

BEGIN

  ts_r_8bit <= RAM_X0_RD_DATA(15 downto 11) & "000";
  ts_g_8bit <= RAM_X0_RD_DATA(10 downto 5) & "00";
  ts_b_8bit <= RAM_X0_RD_DATA(4 downto 0) & "000";

-------------------------------------------------------------------
RAM_X0_0 : SRAM_1024x16 port map (
   WRCLK		=> CLK100M,
   RDCLK		=> CLK100M,
   WR_N	=> RAM_X0_WR_N_0, 
   WRADDR	=> RAM_X0_WR_ADDR, 
   RDADDR	=> RAM_X0_RD_ADDR, 
   WRDATA	=> RAM_X0_WR_DATA, 
   RDDATA	=> RAM_X0_RD_DATA_0 );

-------------------------------------------------------------------
RAM_X0_1 : SRAM_1024x16 port map (
   WRCLK		=> CLK100M,
   RDCLK		=> CLK100M,
   WR_N	=> RAM_X0_WR_N_1, 
   WRADDR	=> RAM_X0_WR_ADDR, 
   RDADDR	=> RAM_X0_RD_ADDR, 
   WRDATA	=> RAM_X0_WR_DATA, 
   RDDATA	=> RAM_X0_RD_DATA_1 );

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     oddline_dly3 <= '0';
     oddline_dly2 <= '0';
     oddline_dly1 <= '0';
   elsif( CLK'event and CLK = '1' ) then
     oddline_dly3 <= oddline_dly2;
     oddline_dly2 <= oddline_dly1;
     oddline_dly1 <= ODDLINE;
   end if;  
end process;

  ODDLINE <= VgaLineCount(1);
  oddline_edge <= '1' when (oddline_dly1 = '0' and oddline_dly2 = '1') else '0';


-------------------------------------------------------------- 
  RAM_X0_WR_N_0 <= '0' when (RAM_X0_WR_SEL = '0' and 
                     (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') and
                     (VgaLinecountMod10 = 0 or VgaLineCountMod10 = 2)
                     ) else '1';
                       
  RAM_X0_WR_N_1 <= '0' when (RAM_X0_WR_SEL = '1' and 
                     (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') and
                     (VgaLinecountMod10 = 0 or VgaLineCountMod10 = 2)
                     ) else '1';
  
  RAM_X0_WR_DATA <= latch0(15 downto 0) when winp0 = '1' else
                        latch1(15 downto 0) when winp1 = '1' else
                        latch2(15 downto 0) when winp2 = '1' else 
                        latch3(15 downto 0);
  
  WRN2DL_sig <= '0' when (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') 
                     else '1';
                       
-------------------------------------------------------------------
  TsLineCount_sig <= "0000" & VgaLineCountMod10(1) & VgaPixCount(8 downto 5);
  TsPixCount_sig <= "00000" & VgaPixCount(4 downto 0);

  TsLineCount <= TsLineCount_sig + CnnBlockV_m30;
  TsPixCount <= TsPixCount_sig + CnnBlockH_m32;

  CnnBlockH_m32 <= (CnnBlockH & "00000");
  CnnBlockV_m30 <= (CnnBlockV & "00000") - (CnnBlockV & '0');

-------------------------------------------------------------- 
process( RST_N, CLK100M )  
begin 
   if( RST_N = '0' ) then
     count4wraddr <= (others => '0');
   elsif( CLK100M'event and CLK100M = '1' ) then
     if(VgaHsync_edge = '1') then 
       count4wraddr <= (others => '0');
     elsif(WRN2DL_sig = '0') then 
       count4wraddr <= count4wraddr + 1;
     end if;
  end if;
end process;

-------------------------------------------------------------- 
  RAM_X0_WR_ADDR <= VgaLineCountMod10(1) & count4wraddr(9 downto 1);

  RAM_X0_WR_SEL <= VgaLineCountDiv10(0); --VgaLineCount(2);

  --RAM_X0_RD_SEL <= not VgaLineCount(2);
  RAM_X0_RD_SEL <= not VgaLineCountDiv10(0);

  RAM_X0_RD_DATA <= RAM_X0_RD_DATA_0 when RAM_X0_RD_SEL = '0' else RAM_X0_RD_DATA_1;

-------------------------------------------------------------------
OutBuf_A : SRAM_1024x16 port map (
   WRCLK		=> CLK,
   RDCLK		=> CLK,
   WR_N	=> OB_WR_N_A, 
   WRADDR	=> OB_WR_ADDR, 
   RDADDR	=> OB_RD_ADDR, 
   WRDATA	=> OB_WR_DATA, 
   RDDATA	=> OB_RD_DATA_A );

-------------------------------------------------------------------
OutBuf_B : SRAM_1024x16 port map (
   WRCLK		=> CLK,
   RDCLK		=> CLK,
   WR_N	=> OB_WR_N_B, 
   WRADDR	=> OB_WR_ADDR, 
   RDADDR	=> OB_RD_ADDR, 
   WRDATA	=> OB_WR_DATA, 
   RDDATA	=> OB_RD_DATA_B );

-------------------------------------------------------------- 
  OB_WR_N_A <= '0' when 
                     (oddline_dly2 = '0' and 
                     (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') 
                     ) else '1';
                       
  OB_WR_N_B <= '0' when 
                     (oddline_dly2 = '1' and 
                     (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1') 
                     ) else '1';
  

  OB_WR_ADDR <= count4wraddr(10 downto 1); -- 2023/06/24
  OB_WR_DATA <= RAM_X0_WR_DATA;

-------------------------------------------------------------- 
--process( RST_N, CLK )  
--begin 
--   if( RST_N = '0' ) then
--     OB_RD_ADDR <= (others => '0');
--     OB_WR_N_A <= '0';
--     OB_WR_N_B <= '0';
--   elsif( CLK'event and CLK = '1' ) then
--     OB_RD_ADDR <= OB_RD_ADDR_sig;
--     OB_WR_N_A <= OB_WR_N_A_sig;
--     OB_WR_N_B <= OB_WR_N_B_sig;
--  end if;
--end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		OB_RD_ADDR <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		OB_RD_ADDR <= VgaPixCount(9 downto 0);
	end if;  
end process;
          
------------------------------------------------
  vga_RGB <= OB_RD_DATA_A when oddline_dly2 = '1' else OB_RD_DATA_B;

  vga_r <= vga_RGB(15 downto 11) & "000";
  vga_g <= vga_RGB(10 downto 5) & "00";
  vga_b <= vga_RGB(4 downto 0) & "000";

--  vga_r_tmp <= vga_RGB(15 downto 11) & "000";
--  vga_g_tmp <= vga_RGB(10 downto 5) & "00";
--  vga_b_tmp <= vga_RGB(4 downto 0) & "000";

--  vga_r <= vga_r_tmp(6 downto 0) & '0' when vga_r_tmp(7) = '0' else "11111111";
--  vga_g <= vga_g_tmp(6 downto 0) & '0' when vga_g_tmp(7) = '0' else "11111111";
--  vga_b <= vga_b_tmp(6 downto 0) & '0' when vga_b_tmp(7) = '0' else "11111111";

END structure;
