-- Display Control
-- Designed by T.Iwata at DIGITALFILTER.COM 2022/03/04

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY VGA_CTRL IS
	PORT	(
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
		CnnStart : out std_logic;
		--- DISPJUDGE : out std_logic;
		VgaLineCount : out std_logic_vector(8 downto 0);
		VgaLineCountDiv10 : out std_logic_vector(5 downto 0);
		VgaLineCountDiv30 : out std_logic_vector(4 downto 0);
		VgaLineCountMod10 : out std_logic_vector(3 downto 0);
		VgaLineCountMod30 : out std_logic_vector(4 downto 0);
		VgaPixCount : out std_logic_vector(9 downto 0);
		VgaPixGate : out std_logic;
		VgaVisible : OUT std_logic;
		VgaVsync : OUT std_logic;
		VgaVsyncDisp : OUT std_logic;
		VgaHsync : OUT std_logic;
		VgaHsyncDisp : OUT std_logic;
		VgaVsync_edge : out std_logic;
		VgaHsync_edge : out std_logic );
	END VGA_CTRL;

ARCHITECTURE structure OF VGA_CTRL IS

	signal VgaLineCount_sig	: std_logic_vector(8 downto 0);
	signal VgaPixCount_sig	: std_logic_vector(9 downto 0);
	signal VgaPixCount_clr : std_logic;
	signal VgaLineCount_clr : std_logic;
	signal VgaPixCount_enb : std_logic;
	signal VgaLineCount_enb : std_logic;
	signal VgaLineCountMod10_sig : std_logic_vector(3 downto 0);
	signal VgaLineCountDiv10_sig : std_logic_vector(5 downto 0);
	signal VgaLineCountDiv30_sig : std_logic_vector(4 downto 0);
	signal VgaLineCountMod10_clr : std_logic;
	signal VgaLineCountDiv10_enb : std_logic;
	signal VgaLineCountDiv30_enb : std_logic;
	signal VgaLineCountMod30_sig : std_logic_vector(4 downto 0);
	signal VgaLineCountMod30_clr : std_logic;
	signal VgaVsync_sig, VgaVsyncDisp_sig : std_logic;
	signal VgaVsync_edge_sig : std_logic;
	signal VgaHsync_sig, VgaHsyncDisp_sig : std_logic;
	signal VgaHsync_edge_sig : std_logic;
	signal VgaFrameCount_enb : std_logic;
	signal VgaVisible_V : std_logic;
	signal VgaVisible_H : std_logic;
	signal count54_clr : std_logic;
	signal count54 : std_logic_vector(5 downto 0);
	signal count300 : std_logic_vector(8 downto 0);
	--signal BigPixN_sig : std_logic_vector(9 downto 0);
	signal WR_N_X_sig : std_logic_vector(2 downto 0);
	signal VgaFrameCount_sig : std_logic_vector(3 downto 0);

BEGIN
-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
  if( RST_N = '0' ) then
    VgaPixCount_enb <= '0';
  elsif( CLK'event and CLK = '1' ) then
    VgaPixCount_enb <= not VgaPixCount_enb;
  end if; 
end process;

  VgaPixGate <= VgaPixCount_enb;

-------------------------------------------------------------- 
   VgaPixCount_clr <= '1' when (VgaPixCount_enb = '1' and VgaPixCount_sig = 799 ) else '0'; 
       -- from 783 2018/07/13
                   -- or CamHsync_EDGE = '1' else '0'; -- commented out 2018/07/13

--WR_ADDR_CLR <= BigPixN_clr;

--  RD_ADDR_LOAD <= '1' when BigLineN_L(1 downto 0) = "11" and VgaLineCount_enb = '1' else '0';

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then -- or SoftReset_N = '0') then
		VgaPixCount_sig <= (others => '1'); -- for soft-reset
	elsif( CLK'event and CLK = '1' ) then
		if(VgaPixCount_clr = '1') then 
			VgaPixCount_sig <= (others => '0');
		elsif(VgaPixCount_enb = '1') then 
			VgaPixCount_sig <= VgaPixCount_sig+ 1;
		end if; 
	end if;  
end process;

  VgaPixCount <= VgaPixCount_sig;

-------------------------------------------------------------- 
   VgaLineCount_enb <= '1' when VgaPixCount_sig = 799 and VgaPixCount_enb = '1' else '0'; -- from 783 2018/07/13
   VgaLineCount_clr <= '1' when VgaPixCount_clr = '1' and VgaLineCount_sig = 499 else '0'; 

   VgaLineCountMod10_clr <= '1' when VgaLineCount_enb = '1' and VgaLineCountMod10_sig = 9 else '0'; -- 2023/05/10
   VgaLineCountDiv10_enb <= VgaLineCountMod10_clr; -- 2023/05/10

   CnnStart <= '1' when VgaLineCountDiv10_enb = '1' or VgaLineCount_clr = '1' else '0';

   VgaLineCountDiv30_enb <= VgaLineCountMod30_clr; -- 2023/06/28

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then -- or SoftReset_N = '0') then
                VgaLineCount_sig <= "111000000"; -- (others => '1'); -- 0x1c0(448) to speed up the simulation
	elsif( CLK'event and CLK = '1' ) then
		if(VgaLineCount_clr = '1') then 
			VgaLineCount_sig <= (others => '0');
		elsif(VgaLineCount_enb = '1') then 
			VgaLineCount_sig <= VgaLineCount_sig+ 1;
		end if; 
	end if;  
end process;

  VgaLineCount <= VgaLineCount_sig;
  
-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
                VgaLineCountDiv10_sig <= (others => '0'); 
	elsif( CLK'event and CLK = '1' ) then
		if(VgaLineCount_clr = '1') then 
			VgaLineCountDiv10_sig <= (others => '0');
		elsif(VgaLineCountDiv10_enb = '1') then 
			VgaLineCountDiv10_sig <= VgaLineCountDiv10_sig + 1;
		end if; 
	end if;  
end process;

  VgaLineCountDiv10 <= VgaLineCountDiv10_sig;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
                VgaLineCountDiv30_sig <= (others => '0'); 
	elsif( CLK'event and CLK = '1' ) then
		if(VgaLineCount_clr = '1') then 
			VgaLineCountDiv30_sig <= (others => '0');
		elsif(VgaLineCountDiv30_enb = '1') then 
			VgaLineCountDiv30_sig <= VgaLineCountDiv30_sig + 1;
		end if; 
	end if;  
end process;

  VgaLineCountDiv30 <= VgaLineCountDiv30_sig;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
                VgaLineCountMod10_sig <= (others => '0'); 
	elsif( CLK'event and CLK = '1' ) then
		if(VgaLineCount_clr = '1' or VgaLineCountMod10_clr = '1') then 
			VgaLineCountMod10_sig <= (others => '0');
		elsif(VgaLineCount_enb = '1') then 
			VgaLineCountMod10_sig <= VgaLineCountMod10_sig + 1;
		end if; 
	end if;  
end process;

  VgaLineCountMod10 <= VgaLineCountMod10_sig;

--------------------------------------------------------------
   VgaLineCountMod30_clr <= '1' when VgaLineCount_enb = '1' and VgaLineCountMod30_sig = 29 else '0'; -- 2023/06/26

process( RST_N, CLK )
begin
        if( RST_N = '0' ) then
                VgaLineCountMod30_sig <= (others => '0');
        elsif( CLK'event and CLK = '1' ) then
                if(VgaLineCount_clr = '1' or VgaLineCountMod30_clr = '1') then
                        VgaLineCountMod30_sig <= (others => '0');
                elsif(VgaLineCount_enb = '1') then
                        VgaLineCountMod30_sig <= VgaLineCountMod30_sig + 1;
                end if;
        end if;
end process;

  VgaLineCountMod30 <= VgaLineCountMod30_sig;

-------------------------------------------------------------- 
   VgaVisible_H <= '1' when VgaPixCount_sig >= 0 and VgaPixCount_sig < 640 else '0'; -- from 5 to 0 2023/06/24 
   VgaHsync_sig <= '1' when VgaPixCount_sig >= 95 else '0'; -- from 96 2019/12/30
   VgaHsyncDisp_sig <= '0' when VgaPixCount_sig >= 650 and VgaPixCount_sig < 650+95 else '1'; -- from 96 2019/12/30
   VgaHsync_edge_sig <= VgaPixCount_clr;
    -- '1' when VgaPixCount_sig = 3 and VgaPixCount_enb = '1' else '0'; -- 2020/01/03
   VgaHsync_edge <= VgaHsync_edge_sig;

-------------------------------------------------------------- 
   VgaVisible_V <= '1'when VgaLineCount_sig >= 0 and VgaLineCount_sig < 480 else '0'; -- 2020/01/05
   VgaVsync_sig <= '0' when VgaLineCount_sig >= 484 and VgaLineCount_sig <= 485 else '1'; -- 2012/11/07
   VgaVsyncDisp_sig <= '0' when VgaLineCount_sig >= 470 and VgaLineCount_sig <= 471 else '1'; -- 2020/01/05
   VgaVsync_edge_sig <= '1' when VgaLineCount_sig = 499 and VgaPixCount_sig = 799 and VgaPixCount_enb = '1' else '0'; -- 2022/01/05
   VgaVsync_edge <= VgaVsync_edge_sig;

-------------------------------------------------------------- 
gen_VgaVisible_sig : process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		VgaVisible <= '0';
	elsif( CLK'event and CLK = '1' ) then
	   if( VgaPixCount_enb = '1' ) then
		   VgaVisible <= VgaVisible_V and VgaVisible_H; 
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
--process( RST_N, CLK )  
--begin 
--	if( RST_N = '0' ) then -- or SoftReset_N = '0') then 
--		VgaFrameCount_sig <= (others => '1'); -- for soft-reset
--	elsif( CLK'event and CLK = '1' ) then
--		if( VgaFrameCount_enb = '1' ) then
--			VgaFrameCount_sig <= VgaFrameCount_sig + 1; 
--		end if;
--	end if;  
--end process;

process( RST_N, VgaVsync_sig )  
begin 
	if( RST_N = '0' ) then -- or SoftReset_N = '0') then 
		VgaFrameCount_sig <= (others => '1'); -- for soft-reset
	elsif( VgaVsync_sig'event and VgaVsync_sig = '1' ) then
			VgaFrameCount_sig <= VgaFrameCount_sig + 1; 
	end if;  
end process;

VgaFrameCount_enb <= '1' when VgaPixCount_enb = '1' and VgaLineCount_enb = '1' 
			and VgaLineCount_sig = 499 else '0';

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		VgaHsync <= '0';
		VgaHsyncDisp <= '0';
	elsif( CLK'event and CLK = '1' ) then
	   if( VgaPixCount_enb = '1' ) then
		   VgaHsync <= VgaHsync_sig; 
		   VgaHsyncDisp <= VgaHsyncDisp_sig; 
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		VgaVsync <= '0';
		VgaVsyncDisp <= '0';
	elsif( CLK'event and CLK = '1' ) then
	   if( VgaPixCount_enb = '1' ) then
		   VgaVsync <= VgaVsync_sig; 
		   VgaVsyncDisp <= VgaVsyncDisp_sig; 
		end if;
	end if;  
end process;

-------------------------------------------------------------- 
-- DISPJUDGE <= '1'when VgaFrameCount_sig(0) = '1' else '0';

END structure;
