library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;
  use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity VideoProc is
   Port ( 
   CLK : In std_logic;
	RST_N	: IN std_logic;  
	RXD : in std_logic;
	TXD : out std_logic;
  SDRAM_DQMH	: out std_logic;
  SDRAM_DQML	: out std_logic;
  SDRAM_CS_N	: out std_logic;
  SDRAM_WE_N	: out std_logic;
  SDRAM_RAS_N	: out std_logic;
  SDRAM_CAS_N	: out std_logic;
  SDRAM_CKE	: out std_logic;
  SDRAM_DQ : inout std_logic_vector(15 downto 0);
  SDRAM_A : out std_logic_vector(12 downto 0);
  SDRAM_BA : out std_logic_vector(1 downto 0);
  SDRAM_CLK : out std_logic;
 	VgaSyncN : out std_logic;
	VgaBlankN : out std_logic;
	VgaClock : out std_logic;
	VgaVsync : OUT std_logic;
	VgaHsync : OUT std_logic;
   VgaDataR	: OUT std_logic_vector(7 downto 0);
   VgaDataG	: OUT std_logic_vector(7 downto 0);
   VgaDataB	: OUT std_logic_vector(7 downto 0);
	SW0 : in std_logic;
	SW1 : in std_logic;
	SW2 : in std_logic;
	LED0 : out std_logic;
	LED1 : out std_logic;
	LED2 : out std_logic;
	LED3 : out std_logic;
	LED4 : out std_logic;
	LED5 : out std_logic;
	LED6 : out std_logic;
	LED7 : out std_logic;
	LED8 : out std_logic;
	LED9 : out std_logic;
	SCL : inout std_logic;
	SDA : inout std_logic;
		PCLK	: IN std_logic;
	CamData	: IN std_logic_vector(7 downto 0);  
	CamHsync	: IN std_logic;  
	CamVsync	: IN std_logic );  
end VideoProc ;

architecture RTL of VideoProc is

component VideoProcCore
	PORT	(
	CLK	: IN std_logic;
	CLK100M	: IN std_logic;
	CLK2DL	: IN std_logic;
	RST_N	: IN std_logic;
	SIMMODE	: IN std_logic;
	THRUMODE	: IN std_logic;
	mdet_enb	: IN std_logic;
  ViewMode : in std_logic_vector(3 downto 0);
	CamHsync	: IN std_logic;
	CamVsync	: IN std_logic;
	PCLK	: IN std_logic;
  CamData	: in std_logic_vector(7 downto 0);
	EMUMODE	: IN std_logic;

	SDRAM_DQENB_N : out std_logic;
  SDRAM_DQMH	: out std_logic;
  SDRAM_DQML	: out std_logic;
  SDRAM_CS_N	: out std_logic;
  SDRAM_WE_N	: out std_logic;
  SDRAM_RAS_N	: out std_logic;
  SDRAM_CAS_N	: out std_logic;
  SDRAM_CKE	: out std_logic;
  SDRAM_DQI : in std_logic_vector(15 downto 0);
  SDRAM_DQO : out std_logic_vector(15 downto 0);
  SDRAM_A : out std_logic_vector(11 downto 0);
  SDRAM_BA : out std_logic_vector(1 downto 0);
  SDRAM_CLK : out std_logic;
	VgaVsync : OUT std_logic;
	VgaVsync_edge : OUT std_logic;
	VgaHsync : OUT std_logic;
	VgaBlankN : OUT std_logic;
  THRESH : in std_logic_vector(13 downto 0); 
  ANSPOS : out std_logic_vector(8 downto 0);
  ANSVALUE : out std_logic_vector(13 downto 0);
  ANSWER : out std_logic_vector(5 downto 0);
                ANS_RD_ADDR_NIOS : in std_logic_vector(8 downto 0);
                ANS_RD_DATA_NIOS : out std_logic_vector(5 downto 0);
   VgaDataR	: OUT std_logic_vector(7 downto 0);
   VgaDataG	: OUT std_logic_vector(7 downto 0);
   VgaDataB	: OUT std_logic_vector(7 downto 0) );
end component;

component mypll 
	PORT
	(
			refclk   : in  std_logic := 'X'; -- clk
			rst      : in  std_logic := 'X'; -- reset
			outclk_0 : out std_logic;        -- clk
			outclk_1 : out std_logic;        -- clk
			outclk_2 : out std_logic;        -- clk
			outclk_3 : out std_logic;        -- clk
			locked   : out std_logic         -- export
	);
end component;

-------------------------------------------------------------------------------
-- Signal
-------------------------------------------------------------------------------
signal counter32b : std_logic_vector(31 downto 0);
signal SDRAM_DQENB_N : std_logic;
signal SDRAM_DQI, SDRAM_DQO : std_logic_vector(15 downto 0);
signal EMUMODE, BINMODE, THRUMODE : std_logic;
signal CLK48M, CLK96M, CLK2DL : std_logic;
signal THRESH : std_logic_vector(13 downto 0);
signal ANSVALUE : std_logic_vector(13 downto 0);
signal ANSWER : std_logic_vector(5 downto 0);
signal ANSPOS : std_logic_vector(8 downto 0);
signal ANSINFO : std_logic_vector(31 downto 0);
signal ANS_RD_ADDR_NIOS : std_logic_vector(8 downto 0);
signal ANS_RD_DATA_NIOS : std_logic_vector(5 downto 0);

begin

  --THRESH <= "00000000100010";

  EMUMODE <= SW0;
  THRUMODE <= SW1;
  BINMODE <= SW2;

  LED0 <= '0';
  LED1 <= counter32b(24);
  LED2 <= '0';
  LED3 <= '0';
  LED4 <= '0';
  LED5 <= '0';
  LED6 <= '0';
  LED7 <= '0';
  LED8 <= '0';
  LED9 <= '0';
  
  VgaSyncN <= '1';-- when VgaVsync = '0' or VgaHsync = '0' else '1';
  VgaClock <= counter32b(0);

  -------------------------------------------------------------- 
gen_counter32b : process( RST_N, CLK48M )  
begin 
	if( RST_N = '0' ) then
		counter32b <= (others => '0');
	elsif( CLK48M'event and CLK48M = '1' ) then
		counter32b <= counter32b + 1;
	end if;  
end process;
  
 VideoProcCore_1 : VideoProcCore port map (
		CLK	=> CLK48M,
		CLK100M	=> CLK96M,
		CLK2DL	=> CLK2DL,
		RST_N	=> RST_N,
     SIMMODE => '0', -- NOT simulation
     THRUMODE => THRUMODE,
		mdet_enb => '1',
		ViewMode => "0000",--ViewMode,
	CamHsync	=> CamHsync,
	CamVsync	=> CamVsync,
	PCLK	=> PCLK,
   CamData	=> CamData,
   EMUMODE => EMUMODE,
  SDRAM_DQENB_N => SDRAM_DQENB_N,
  SDRAM_DQMH	=> SDRAM_DQMH,
  SDRAM_DQML	=> SDRAM_DQML,
  SDRAM_CS_N	=> SDRAM_CS_N,
  SDRAM_WE_N	=> SDRAM_WE_N,
  SDRAM_RAS_N	=> SDRAM_RAS_N,
  SDRAM_CAS_N	=> SDRAM_CAS_N,
  SDRAM_CKE	=> SDRAM_CKE,
  SDRAM_DQI => SDRAM_DQI,
  SDRAM_DQO => SDRAM_DQO,
  SDRAM_A => SDRAM_A(11 downto 0),
  SDRAM_BA => SDRAM_BA,
  SDRAM_CLK	=> open,
      VgaVsync => VgaVsync,
      VgaVsync_edge => open,
      VgaHsync => VgaHsync,
      VgaBlankN => VgaBlankN,
  THRESH => THRESH, 
  ANSPOS => ANSPOS, 
  ANSVALUE => ANSVALUE, 
  ANSWER => ANSWER, 
  ANS_RD_ADDR_NIOS => ANS_RD_ADDR_NIOS, 
  ANS_RD_DATA_NIOS => ANS_RD_DATA_NIOS, 
      VgaDataR => VgaDataR, 
		VgaDataG => VgaDataG, 
		VgaDataB => VgaDataB );

------------------------------------------------------------- 
  SDRAM_DQI <= SDRAM_DQ;
  SDRAM_DQ <= SDRAM_DQO when SDRAM_DQENB_N = '0' else (others => 'Z');
  SDRAM_A(12) <= '0';

  -------------------------------------------------------------- 
		
		ANSINFO <= ANSPOS & "0" & ANSVALUE & "00" & ANSWER;
  -------------------------------------------------------------- 
  mypll_1 : mypll 
    port map(
		refclk	=> CLK,
		rst	=> '0',
		outclk_0		=> CLK48M,
		outclk_1		=> CLK96M,
		outclk_2		=> SDRAM_CLK,
		outclk_3		=> CLK2DL,
		locked	=> open );
		
end RTL;
