-- Deep Learning Top Module
-- Designed by T.Iwata at DIGITALFILTER.COM 2023/06/28

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity VideoProcCore is
	PORT	(
	CLK	: IN std_logic;
	CLK100M	: IN std_logic;
	CLK2DL	: IN std_logic;
	RST_N	: IN std_logic;
	SIMMODE	: IN std_logic;
	THRUMODE	: IN std_logic;
	mdet_enb	: IN std_logic;
  ViewMode : in std_logic_vector(3 downto 0);
	CamHsync	: IN std_logic;
	CamVsync	: IN std_logic;
	PCLK	: IN std_logic;
  CamData	: in std_logic_vector(7 downto 0);
  	EMUMODE	: IN std_logic;

	SDRAM_DQENB_N : out std_logic;
  SDRAM_DQMH	: out std_logic;
  SDRAM_DQML	: out std_logic;
  SDRAM_CS_N	: out std_logic;
  SDRAM_WE_N	: out std_logic;
  SDRAM_RAS_N	: out std_logic;
  SDRAM_CAS_N	: out std_logic;
  SDRAM_CKE	: out std_logic;
  SDRAM_DQI : in std_logic_vector(15 downto 0);
  SDRAM_DQO : out std_logic_vector(15 downto 0);
  SDRAM_A : out std_logic_vector(11 downto 0);
  SDRAM_BA : out std_logic_vector(1 downto 0);
  SDRAM_CLK : out std_logic;
	VgaVsync : OUT std_logic;
	VgaVsync_edge : OUT std_logic;
	VgaHsync : OUT std_logic;
	VgaBlankN : OUT std_logic;
  THRESH : in std_logic_vector(13 downto 0);
  ANSPOS : out std_logic_vector(8 downto 0);
  ANSVALUE : out std_logic_vector(13 downto 0);
                ANS_RD_ADDR_NIOS : in std_logic_vector(8 downto 0);
                ANS_RD_DATA_NIOS : out std_logic_vector(5 downto 0);
   ANSWER	: OUT std_logic_vector(5 downto 0);
   VgaDataR	: OUT std_logic_vector(7 downto 0);
   VgaDataG	: OUT std_logic_vector(7 downto 0);
   VgaDataB	: OUT std_logic_vector(7 downto 0) );
end VideoProcCore ;

architecture structure of VideoProcCore is

component CAM_CTRL
	PORT	(
		RST_N	: IN std_logic;
		PCLK	: IN std_logic;
		CamHsync	: IN std_logic;
		CamVsync	: IN std_logic;
		CamData	: IN std_logic_vector(7 downto 0);
		LB_WR_ADDR	: out std_logic_vector(9 downto 0);
		LB_WR_DATA	: out std_logic_vector(15 downto 0);
		LB_WR_N	: out std_logic;
		CamLineCount30 : out std_logic_vector(4 downto 0);
		CamPixCount : out std_logic_vector(10 downto 0); 
		CamLineCount : out std_logic_vector(8 downto 0)); 
end component;

component VGA_CTRL
	PORT	(
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
		CnnStart	: out std_logic;
		--- DISPJUDGE	: out std_logic;
		VgaPixCount : out std_logic_vector(9 downto 0);
		VgaLineCount	: out std_logic_vector(8 downto 0);
                VgaLineCountDiv10 : out std_logic_vector(5 downto 0);
                VgaLineCountDiv30 : out std_logic_vector(4 downto 0);
                VgaLineCountMod10 : out std_logic_vector(3 downto 0);
                VgaLineCountMod30 : out std_logic_vector(4 downto 0);
		VgaPixGate : out std_logic;
		VgaVisible : OUT std_logic;
		VgaVsync : OUT std_logic;
		VgaVsyncDisp : OUT std_logic;
		VgaHsync : OUT std_logic;
		VgaHsyncDisp : OUT std_logic;
		VgaVsync_edge : out std_logic;
		VgaHsync_edge : out std_logic );
end component;

component LineIn_CTRL
	Port (
		PCLK	: IN std_logic;
		CLK	: IN std_logic;
		RST_N	: IN std_logic;
		LB_WR_ADDR	: IN std_logic_vector(9 downto 0);
		LB_WR_DATA	: IN std_logic_vector(15 downto 0);
		LB_WR_N	: IN std_logic;
		CamLineCount : in std_logic_vector(8 downto 0); 
		CamLineCount_sync : out std_logic_vector(8 downto 0); 
		VgaLineCount : in std_logic_vector(8 downto 0);
		VgaPixCount : in std_logic_vector(9 downto 0);
		VgaHsync_edge : in std_logic;
		buf_RGB : out std_logic_vector(15 downto 0)
);
end component;

component LineOut_CTRL
	Port (
		CLK	: IN std_logic;
		CLK100M	: IN std_logic;
		RST_N	: IN std_logic;
  TsLineCount : out std_logic_vector(8 downto 0);
  TsPixCount : out std_logic_vector(9 downto 0);
                CnnBlockH : in std_logic_vector(4 downto 0);
                CnnBlockV : in std_logic_vector(3 downto 0);
VgaHsync_edge : in std_logic;
                RAM_X0_RD_ADDR : in std_logic_vector(9 downto 0);
                ts_r_8bit : out std_logic_vector(7 downto 0);
                ts_g_8bit : out std_logic_vector(7 downto 0);
                ts_b_8bit : out std_logic_vector(7 downto 0);
		latch0	: IN std_logic_vector(15 downto 0);
		latch1	: IN std_logic_vector(15 downto 0);
		latch2	: IN std_logic_vector(15 downto 0);
		latch3	: IN std_logic_vector(15 downto 0);
		VgaLineCount	: in std_logic_vector(8 downto 0);
                VgaLineCountDiv10 : in std_logic_vector(5 downto 0);
                VgaLineCountMod10 : in std_logic_vector(3 downto 0);
		VgaPixCount : IN std_logic_vector(9 downto 0);
		winp0: in std_logic;
		winp1: in std_logic;
		winp2: in std_logic;
		winp3: in std_logic;
   vga_r : out std_logic_vector(7 downto 0);
   vga_g : out std_logic_vector(7 downto 0);
   vga_b : out std_logic_vector(7 downto 0) );
end component;

component SDRAM_CTRL
	Port (
		CLK	: IN std_logic;
		CLK100M	: IN std_logic;
		RST_N	: IN std_logic;
  TsLineCount : in std_logic_vector(8 downto 0);
  TsPixCount : in std_logic_vector(9 downto 0);
DISPJUDGE : in std_logic;
		mdet_enb	: IN std_logic;
  ViewMode : in std_logic_vector(3 downto 0);
		VgaHsync_edge : in std_logic;
      VgaVsync_edge : in std_logic;
		CamVsync : in std_logic;
		CamLineCount: in std_logic_vector(8 downto 0);
		VgaPixCount : in std_logic_vector(9 downto 0);
		VgaPixGate : in std_logic;
		VgaLineCount	: in std_logic_vector(8 downto 0);
  buf_RGB : in std_logic_vector(15 downto 0);
		latch0	: out std_logic_vector(15 downto 0);
		latch1	: out std_logic_vector(15 downto 0);
		latch2	: out std_logic_vector(15 downto 0);
		latch3	: out std_logic_vector(15 downto 0);
		winp0: out std_logic;
		winp1: out std_logic;
		winp2: out std_logic;
		winp3: out std_logic;
  SDRAM_DQENB_N: out std_logic;
  SDRAM_DQMH	: out std_logic;
  SDRAM_DQML	: out std_logic;
  SDRAM_CS_N	: out std_logic;
  SDRAM_WE_N	: out std_logic;
  SDRAM_RAS_N	: out std_logic;
  SDRAM_CAS_N	: out std_logic;
  SDRAM_CKE	: out std_logic;
  SDRAM_DQI : in std_logic_vector(15 downto 0);
  SDRAM_DQO : out std_logic_vector(15 downto 0);
  SDRAM_A : out std_logic_vector(11 downto 0);
  SDRAM_BA : out std_logic_vector(1 downto 0) );
end component;

signal VgaHsync_sig, VgaHsyncDisp : std_logic;
signal 	LB_WR_ADDR	: std_logic_vector(9 downto 0);
signal 	LB_WR_DATA	: std_logic_vector(15 downto 0);
signal 	LB_WR_N	: std_logic;
signal CamHsync_EDGE, CamVsync_EDGE : std_logic;
signal VgaVisible : std_logic;
 signal buf_RGB : std_logic_vector(15 downto 0);
signal VgaHsync_edge : std_logic;
signal VgaVsync_edge_sig : std_logic;
signal CamLineCount : std_logic_vector(8 downto 0);
signal CamLineCount_sync : std_logic_vector(8 downto 0);
signal latch0, latch1, latch2, latch3 : std_logic_vector(15 downto 0);
signal winp0,winp1,winp2,winp3 : std_logic;
signal  vga_RGB_latch0 : std_logic_vector(15 downto 0);
signal  vga_RGB_latch1 : std_logic_vector(15 downto 0);
signal  vga_RGB_latch2 : std_logic_vector(15 downto 0);
signal  vga_RGB_latch3 : std_logic_vector(15 downto 0);
signal  counter0 : std_logic_vector(23 downto 0);
signal  counter0_clr, counter0_dec : std_logic;
signal VgaPixCount : std_logic_vector(9 downto 0);
signal VgaLineCount	: std_logic_vector(8 downto 0);
signal VgaLineCountDiv10 : std_logic_vector(5 downto 0);
signal VgaLineCountDiv30 : std_logic_vector(4 downto 0);
signal VgaLineCountMod10 : std_logic_vector(3 downto 0);
signal VgaLineCountMod30 : std_logic_vector(4 downto 0);
 signal vga_r, vga_g, vga_b : std_logic_vector(7 downto 0);
signal LBRDY_sig : std_logic;
signal VgaPixGate : std_logic;
signal TsLineCount : std_logic_vector(8 downto 0);
signal TsPixCount : std_logic_vector(9 downto 0);
signal CAM_RD_ADDR : std_logic_vector(15 downto 0);
signal CAM_RD_DATA : std_logic_vector(7 downto 0);
signal CamPixCount : std_logic_vector(10 downto 0); 
signal CAM_CS_N : std_logic;
signal CamLineCount30 : std_logic_vector(4 downto 0);
signal RD_ADDR_LOAD, RD_ADDR_ENB, WR_ADDR_CLR : std_logic;
signal CO300 : std_logic_vector(8 downto 0);
signal RD_BUF_SEL, WR_BUF_SEL : std_logic_vector(1 downto 0);
signal RD_BUF_ADDR : std_logic_vector(8 downto 0);
signal WR_BUF_N : std_logic;
signal RAM_X0_RD_ADDR : std_logic_vector(9 downto 0);
signal ts_r_8bit, ts_g_8bit, ts_b_8bit : std_logic_vector(7 downto 0);
signal CnnStart : std_logic;
signal CnnBlockH : std_logic_vector(4 downto 0);
signal CnnBlockV : std_logic_vector(3 downto 0);
signal CamDataMix : std_logic_vector(7 downto 0);
signal CamData_dly : std_logic_vector(7 downto 0);
signal vga_r_deep, vga_g_deep, vga_b_deep : std_logic_vector(7 downto 0);
signal DISPJUDGE : std_logic;
signal DISPJUDGE_mask : std_logic;

begin

  DISPJUDGE_mask <= '0';
--  DISPJUDGE_mask <= '0' when THRUMODE = '1' -- always no judge, (through)
--               else '1' when SIMMODE = '1' -- always judge, (for simulation)
--               else DISPJUDGE; 

  VgaHsync <= VgaHsyncDisp;

  VgaDataR <= "00000000" when VgaVisible = '0' else 
--		vga_r_deep when DISPJUDGE_mask = '1' else 
		vga_r;

  VgaDataG <= "00000000" when VgaVisible = '0' else 
--		vga_g_deep when DISPJUDGE_mask = '1' else 
		vga_g;

  VgaDataB <= "00000000" when VgaVisible = '0' else 
--		vga_b_deep when DISPJUDGE_mask = '1' else 
		vga_b;

  VgaBlankN <= VgaVisible;

  
  CamDataMix <= CamData_dly;-- when EMUMODE = '0' else CAM_RD_DATA;
--  CamDataMix <= CamData_dly;--CAM_RD_DATA;

--------------------------------------------------------------
process( RST_N, PCLK ) 
begin
   if( RST_N = '0' ) then
     CamData_dly <= (others => '0');
   elsif( PCLK'event and PCLK = '1' ) then
     CamData_dly <= CamData;
   end if;
end process;

---------------------------------------------------------
CAM_CTRL_1 : CAM_CTRL port map (
		RST_N	=> RST_N,
		PCLK	=> PCLK,
		CamHsync	=> CamHsync,
		CamVsync	=> CamVsync,
		CamData	=> CamDataMix,
		LB_WR_ADDR	=> LB_WR_ADDR,
		LB_WR_DATA	=> LB_WR_DATA,
		LB_WR_N	=> LB_WR_N,
		CamLineCount30 => CamLineCount30,
		CamPixCount => CamPixCount, 
		CamLineCount => CamLineCount ); 

VGA_CTRL_1 : VGA_CTRL port map (
		CLK	=> CLK,
		RST_N	=> RST_N,
		CnnStart	=> CnnStart, 
		--- DISPJUDGE	=> DISPJUDGE_sig, 
		VgaLineCount	=> VgaLineCount, 
		VgaLineCountDiv10 => VgaLineCountDiv10, 
		VgaLineCountDiv30 => VgaLineCountDiv30, 
		VgaLineCountMod10 => VgaLineCountMod10, 
		VgaLineCountMod30 => VgaLineCountMod30, 
		VgaPixCount	=> VgaPixCount, 
		VgaPixGate => VgaPixGate, 
		VgaVisible => VgaVisible,
		VgaVsync => open,
		VgaVsyncDisp => VgaVsync,
		VgaHsync => VgaHsync_sig,
		VgaHsyncDisp => VgaHsyncDisp,
		VgaVsync_edge => VgaVsync_edge_sig,
		VgaHsync_edge => VgaHsync_edge );

LineIn_CTRL_1 : LineIn_CTRL port map (
   PCLK		=> PCLK,
   CLK		=> CLK,
		RST_N	=> RST_N,
		LB_WR_ADDR	=> LB_WR_ADDR,
		LB_WR_DATA	=> LB_WR_DATA,
		LB_WR_N	=> LB_WR_N,
		CamLineCount => CamLineCount,
		CamLineCount_sync => CamLineCount_sync,
		VgaLineCount => VgaLineCount,
		VgaPixCount	=> VgaPixCount, 
		VgaHsync_edge => VgaHsync_edge,
		buf_RGB => buf_RGB );

LineOut_CTRL_1 : LineOut_CTRL port map (
   CLK		=> CLK,
   CLK100M		=> CLK100M,
		RST_N	=> RST_N,
  TsLineCount => TsLineCount, 
  TsPixCount => TsPixCount, 
  CnnBlockH => CnnBlockH,
  CnnBlockV => CnnBlockV,
		VgaHsync_edge => VgaHsync_edge,
		RAM_X0_RD_ADDR => RAM_X0_RD_ADDR,
   ts_r_8bit => ts_r_8bit,
   ts_g_8bit => ts_g_8bit,
   ts_b_8bit => ts_b_8bit,
		latch0	=> latch0,
		latch1	=> latch1,
		latch2	=> latch2,
		latch3	=> latch3,
		VgaLineCount	=> VgaLineCount, 
		VgaLineCountDiv10 => VgaLineCountDiv10, 
		VgaLineCountMod10 => VgaLineCountMod10, 
		VgaPixCount	=> VgaPixCount,
		winp0	=> winp0,
		winp1	=> winp1,
		winp2	=> winp2,
		winp3	=> winp3,
  vga_r => vga_r, 
  vga_g => vga_g,
  vga_b => vga_b	);

SDRAM_CTRL_1 : SDRAM_CTRL port map (
   CLK		=> CLK,
   CLK100M		=> CLK100M,
		RST_N	=> RST_N,
		TsLineCount => TsLineCount,
		TsPixCount => TsPixCount,
		DISPJUDGE	=> DISPJUDGE_mask,
		mdet_enb	=> mdet_enb,
		ViewMode => ViewMode,
		VgaHsync_edge => VgaHsync_edge,
		VgaVsync_edge => VgaVsync_edge_sig,
		CamVsync => CamVsync,
		CamLineCount => CamLineCount_sync, -- 2023/06/23
		VgaPixCount => VgaPixCount,
		VgaPixGate	=> VgaPixGate, 
		VgaLineCount	=> VgaLineCount, 
  buf_RGB => buf_RGB, 
		latch0	=> latch0,
		latch1	=> latch1,
		latch2	=> latch2,
		latch3	=> latch3,
		winp0	=> winp0,
		winp1	=> winp1,
		winp2	=> winp2,
		winp3	=> winp3,
  SDRAM_DQENB_N => SDRAM_DQENB_N,
  SDRAM_DQMH	=> SDRAM_DQMH,
  SDRAM_DQML	=> SDRAM_DQML,
  SDRAM_CS_N	=> SDRAM_CS_N,
  SDRAM_WE_N	=> SDRAM_WE_N,
  SDRAM_RAS_N	=> SDRAM_RAS_N,
  SDRAM_CAS_N	=> SDRAM_CAS_N,
  SDRAM_CKE	=> SDRAM_CKE,
  SDRAM_DQI => SDRAM_DQI,
  SDRAM_DQO => SDRAM_DQO,
  SDRAM_A => SDRAM_A,
  SDRAM_BA => SDRAM_BA
 );

  SDRAM_CLK <= CLK100M;

  VgaVsync_edge <= VgaVsync_edge_sig; 
  
end structure ;
